require 'spec_helper'

RSpec.describe TikaMasala::Parser do
  subject { described_class.new(tika_jar) }

  let(:tika_jar) { jar_path('tika-app-1.5.jar') }
  let(:pdf_file) { fixtures_path('test.pdf') }
  let(:word_file) { fixtures_path('test.docx') }

  it { respond_to(:parse) }

  describe 'initialize' do
    subject { described_class }

    let(:fake_path) { '/no/such/file/exist' }

    it 'raises an exception if the file does not exist' do
      expect {
        subject.new(fake_path)
      }.to raise_error TikaMasala::FileNotFound, "Jar #{fake_path} does not exist"
    end

    context 'when no parameters are passed' do
      it 'sets the jar to the one in our dependencies folder' do
        expect(subject.new.jar_path).to eq(jar_path('tika-app-1.5.jar'))
      end
    end
  end

  describe '#parse' do
    it 'parses pdf files properly' do
      expect(subject.parse(pdf_file)).to match(/hello world/i)
    end

    it 'parses word files properly' do
      expect(subject.parse(word_file)).to match(/hello world/i)
    end

    context 'when the file does not exist' do
      it 'raises an exception' do
        expect { subject.parse(fixtures_path('file-dont-exist.ppt')) }.to raise_error(TikaMasala::TikaError)
      end
    end

    context 'when the file url contains reserved shell words' do
      let(:file_path) { 'http://example.com/?token=123&param=a' }
      let(:file_path_escaped) { 'http://example.com/\\?token\\=123\\&param\\=a' }
      let(:popen_result_double) do
        [
          double('stdin', close: true),
          double('stdout', close: true, read: ''),
          double('stderr', close: true),
          double('wait_thr', value: 0)
        ]
      end

      it 'escapes the string properly' do
        expect(Open3).to receive(:popen3).with("java -jar #{tika_jar} --text #{file_path_escaped}") do
          popen_result_double
        end

        subject.parse(file_path)
      end
    end
  end

  describe '#metadata' do
    it 'checks the metadata from the file' do
      expect(subject.metadata(word_file)).to match(/Content-Type.*word/i)
    end
  end

  describe '#detect_type' do
    it 'finds the file type' do
      expect(subject.detect_type(pdf_file)).to match(/application.pdf/i)
    end
  end

  describe '#java_exec' do
    it 'takes option directly to tika' do
      expect(subject.java_exec('--text', word_file)).to match(/hello world/i)
    end

    context 'when exit code is not zero' do
      let(:command) { "echo '#{normal_message}'; echo '#{error_message}' 1>&2; exit 1;" }
      let(:error_message) { 'Some nasty error occured' }
      let(:normal_message) { 'Hello world' }

      before do
        popen_result = Open3.popen3(command)

        allow(Open3).to receive(:popen3) do |*args, &block|
          if block_given?
            block.call(*popen_result)
          else
            popen_result
          end
        end
      end

      it 'raises an exception containing stderr' do
        expect { subject.java_exec }.to raise_error(TikaMasala::TikaError, /#{error_message}/)
      end

      describe 'the expection raised' do
        let(:returned_expection) do
          [].tap do |returned_expection|
            begin
              subject.java_exec
            rescue TikaMasala::TikaError => e
              returned_expection << e
            end
          end.first
        end

        it 'does not contain the normal message' do
          expect(returned_expection.message).to_not include(normal_message)
        end

        it 'provides another attribute which contains the ouput of stdout' do
          expect(returned_expection.stdout).to include(normal_message)
        end
      end
    end
  end
end
